@@ -2,21 +2,24 @@
 
 use core::fmt::{self, Display, Formatter};
 use core::{char::DecodeUtf16Error, str::EncodeUtf16, str::Utf8Error};
+#[cfg(feature = "logs")]
+use log::trace;
 
-/// Every error possible when using the [`ArrayString`]
+/// Every error possible when using [`ArrayString`]
 ///
-/// [`ArrayString`]: ./array/trait.ArrayString.html
+/// [`ArrayString`]: ../struct.ArrayString.html
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug)]
 pub enum Error {
-    /// Conversion from byte slice to UTF-8 failed (invalid data or invalid index)
+    /// Conversion between available byte slice and UTF-8 failed (invalid data or invalid utf-8 character index)
     Utf8,
-    /// Conversion from `u16` slice to string failed
+    /// Conversion between available `u16` slice and string failed
     Utf16,
-    /// Out of bounds access
+    /// Out of boundaries access
     OutOfBounds,
 }
 
 impl Display for Error {
+    #[inline]
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         match self {
             Error::Utf8 => write!(f, "Utf8"),
@@ -26,22 +29,25 @@ impl Display for Error {
     }
 }
 
-#[cfg(features = "std")]
+#[cfg(feature = "std")]
 impl std::error::Error for Error {}
 
 impl From<Utf8Error> for Error {
+    #[inline]
     fn from(_: Utf8Error) -> Self {
         Error::Utf8
     }
 }
 
 impl From<DecodeUtf16Error> for Error {
+    #[inline]
     fn from(_: DecodeUtf16Error) -> Self {
         Error::Utf16
     }
 }
 
 impl<'a> From<EncodeUtf16<'a>> for Error {
+    #[inline]
     fn from(_: EncodeUtf16) -> Self {
         Error::Utf16
     }
@@ -52,15 +58,17 @@ impl<'a> From<EncodeUtf16<'a>> for Error {
 pub struct Utf8;
 
 impl Display for Utf8 {
+    #[inline]
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         write!(f, "Utf8")
     }
 }
 
-#[cfg(features = "std")]
+#[cfg(feature = "std")]
 impl std::error::Error for Utf8 {}
 
 impl From<Utf8Error> for Utf8 {
+    #[inline]
     fn from(_: Utf8Error) -> Self {
         Utf8
     }
@@ -78,10 +86,11 @@ impl From<Utf8> for Error {
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug, Default)]
 pub struct Utf16;
 
-#[cfg(features = "std")]
+#[cfg(feature = "std")]
 impl std::error::Error for Utf16 {}
 
 impl Display for Utf16 {
+    #[inline]
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         write!(f, "Utf16")
     }
@@ -96,28 +105,33 @@ impl From<Utf16> for Error {
 }
 
 impl From<DecodeUtf16Error> for Utf16 {
+    #[inline]
     fn from(_: DecodeUtf16Error) -> Self {
         Utf16
     }
 }
 
 impl<'a> From<EncodeUtf16<'a>> for Utf16 {
+    #[inline]
     fn from(_: EncodeUtf16) -> Self {
         Utf16
     }
 }
 
-/// Error caused by out of bounds access to `LimitedString`
+/// Error caused by out of bounds access to [`ArrayString`]
+///
+/// [`ArrayString`]: ../struct.ArrayString.html
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug, Default)]
 pub struct OutOfBounds;
 
 impl Display for OutOfBounds {
+    #[inline]
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         write!(f, "OutOfBounds")
     }
 }
 
-#[cfg(features = "std")]
+#[cfg(feature = "std")]
 impl std::error::Error for OutOfBounds {}
 
 impl From<OutOfBounds> for Error {
