@@ -1,24 +1,16 @@
 //! Misc functions to improve readability
 
+use crate::{array::ArrayString, prelude::*};
 use core::ptr::copy;
-use prelude::*;
-
-/// Setup `env_logger`
-#[cfg(all(feature = "logs", feature = "std"))]
-pub fn setup_logger() {
-    use std::sync::Once;
-    static INITIALIZE: Once = Once::new();
-    INITIALIZE.call_once(env_logger::init);
-}
-
-/// Mocks `setup_logger`
-#[cfg(any(not(feature = "logs"), not(feature = "std")))]
-#[doc(hidden)]
-pub fn setup_logger() {}
+use generic_array::ArrayLength;
+#[cfg(feature = "logs")]
+use log::{debug, trace};
+use generic_array::typenum::Unsigned;
 
 /// Marks branch as impossible, UB if taken in prod, panics in debug
 ///
 /// This function should never be used lightly, it will cause UB if used wrong
+#[inline]
 #[allow(unused_variables)]
 pub(crate) unsafe fn never(s: &str) -> ! {
     #[cfg(debug_assertions)]
@@ -40,7 +32,11 @@ pub(crate) unsafe fn never(s: &str) -> ! {
 /// - It's UB if index is outside of buffer's boundaries (buffer needs at most 4 bytes)
 /// - It's UB if index is inside a character (like a index 3 for "a🤔")
 #[inline]
-pub(crate) unsafe fn encode_char_utf8_unchecked<S: ArrayString>(s: &mut S, ch: char, index: Size) {
+pub(crate) unsafe fn encode_char_utf8_unchecked<S: ArrayLength<u8>>(
+    s: &mut ArrayString<S>,
+    ch: char,
+    index: u8,
+) {
     trace!("Encode char: {} to {}", ch, index);
 
     // UTF-8 ranges and tags for encoding characters
@@ -52,12 +48,17 @@ pub(crate) unsafe fn encode_char_utf8_unchecked<S: ArrayString>(s: &mut S, ch: c
     const MAX_TWO_B: u32 = 0x800;
     const MAX_THREE_B: u32 = 0x10000;
 
-    debug_assert!(index + ch.len_utf8() as Size <= S::CAPACITY);
-    debug_assert!(s.len() + ch.len_utf8() as Size <= S::CAPACITY);
-    let (dst, code) = (s.buffer().get_unchecked_mut(index as usize..), ch as u32);
+    debug_assert!(ch.len_utf8().saturating_add(index.into()) <= <S as Unsigned>::to_u8() as usize);
+    debug_assert!(
+        ch.len_utf8().saturating_add(s.len().into()) <= <S as Unsigned>::to_u8() as usize
+    );
+    let (dst, code) = (
+        s.array.as_mut_slice().get_unchecked_mut(index.into()..),
+        ch as u32,
+    );
 
     if code < MAX_ONE_B {
-        debug_assert!(dst.len() >= 1);
+        debug_assert!(!dst.is_empty());
         *dst.get_unchecked_mut(0) = code as u8;
     } else if code < MAX_TWO_B {
         debug_assert!(dst.len() >= 2);
@@ -77,61 +78,85 @@ pub(crate) unsafe fn encode_char_utf8_unchecked<S: ArrayString>(s: &mut S, ch: c
     }
 }
 
+/// Copies part of slice to another part (`mem::copy`, basically `memmove`)
 #[inline]
 unsafe fn shift_unchecked(s: &mut [u8], from: usize, to: usize, len: usize) {
-    debug_assert!(to + len <= s.len() && from + len <= s.len());
-    debug!("Shift {}, {} to {}", &s[from..from + len], from, to);
-    let (from, to) = (s.as_ptr().add(from), s.as_mut_ptr().add(to));
-    copy(from, to, len);
+    debug!(
+        "Shift {:?}, {} to {}",
+        &s.get(from..from.saturating_add(len)),
+        from,
+        to
+    );
+    debug_assert!(to.saturating_add(len) <= s.len() && from.saturating_add(len) <= s.len());
+    let (f, t) = (s.as_ptr().add(from), s.as_mut_ptr().add(to));
+    copy(f, t, len);
 }
 
+/// Shifts string right
 /// # Safety
 ///
-/// It's UB if `to + (s.len() - from)` is out of [`S::CAPACITY`]
+/// It's UB if `to + (s.len() - from)` is out of [`<S as Unsigned>::to_u8()`]
 ///
-/// [`S::CAPACITY`]: ../array/trait.ArrayString.html#CAPACITY
+/// [`<S as Unsigned>::to_u8()`]: ../struct.ArrayString.html#CAPACITY
 #[inline]
-pub(crate) unsafe fn shift_right_unchecked<S: ArrayString>(s: &mut S, from: Size, to: Size) {
-    let (from, to, len) = (from as usize, to as usize, s.len() - from);
-    debug_assert!(from <= to && to + len as usize <= S::CAPACITY as usize);
-    shift_unchecked(s.buffer(), from, to, len as usize);
+pub(crate) unsafe fn shift_right_unchecked<S: ArrayLength<u8>>(
+    s: &mut ArrayString<S>,
+    from: u8,
+    to: u8,
+) {
+    let (f, t, l) = (from as usize, to as usize, s.len().saturating_sub(from));
+    debug_assert!(f <= t && t.saturating_add(l.into()) <= <S as Unsigned>::to_u8() as usize);
+    debug_assert!(s.as_str().is_char_boundary(f));
+    shift_unchecked(s.array.as_mut_slice(), f, t, l.into());
 }
 
+/// Shifts string left
 #[inline]
-pub(crate) unsafe fn shift_left_unchecked<S: ArrayString>(s: &mut S, from: Size, to: Size) {
+pub(crate) unsafe fn shift_left_unchecked<S: ArrayLength<u8>>(
+    s: &mut ArrayString<S>,
+    from: u8,
+    to: u8,
+) {
     debug_assert!(to <= from && from <= s.len());
-    let (from, to, len) = (from as usize, to as usize, s.len() - to);
-    shift_unchecked(s.buffer(), from, to, len as usize);
+    let (f, t, l) = (from as usize, to as usize, s.len().saturating_sub(to));
+    debug_assert!(s.as_str().is_char_boundary(f));
+    shift_unchecked(s.array.as_mut_slice(), f, t, l.into());
 }
 
 /// Returns error if size is outside of specified boundary
 #[inline]
-pub fn is_inside_boundary(size: Size, limit: Size) -> Result<(), OutOfBounds> {
-    trace!("Out of bounds: ensures {} < {}", size, limit);
-    Some(()).filter(|_| size <= limit).ok_or(OutOfBounds)
+pub fn is_inside_boundary<S, L>(size: S, limit: L) -> Result<(), OutOfBounds>
+where
+    S: Into<usize>,
+    L: Into<usize>,
+{
+    let (s, l) = (size.into(), limit.into());
+    trace!("Out of bounds: ensures {} < {}", s, l);
+    Some(()).filter(|_| s <= l).ok_or(OutOfBounds)
 }
 
 /// Returns error if index is not at a valid utf-8 char boundary
 #[inline]
-pub fn is_char_boundary<S: ArrayString>(s: &S, idx: Size) -> Result<(), Utf8> {
+pub fn is_char_boundary<S: ArrayLength<u8>>(s: &ArrayString<S>, idx: u8) -> Result<(), Utf8> {
     trace!("Is char boundary: {} at {}", s.as_str(), idx);
-    if s.as_str().is_char_boundary(idx as usize) {
+    if s.as_str().is_char_boundary(idx.into()) {
         return Ok(());
     }
     Err(Utf8)
 }
 
+/// Truncates string to specified size (ignoring last bytes if they form a partial `char`)
 #[inline]
-pub(crate) fn truncate_str(slice: &str, size: Size) -> &str {
+pub(crate) fn truncate_str(slice: &str, size: u8) -> &str {
     trace!("Truncate str: {} at {}", slice, size);
-    if slice.is_char_boundary(size as usize) {
-        unsafe { &slice.get_unchecked(..size as usize) }
+    if slice.is_char_boundary(size.into()) {
+        unsafe { slice.get_unchecked(..size.into()) }
     } else if (size as usize) < slice.len() {
         let mut index = size.saturating_sub(1) as usize;
         while !slice.is_char_boundary(index) {
             index = index.saturating_sub(1);
         }
-        unsafe { &slice.get_unchecked(..index) }
+        unsafe { slice.get_unchecked(..index) }
     } else {
         slice
     }
@@ -142,18 +167,8 @@ mod tests {
     #[cfg(feature = "logs")]
     extern crate env_logger;
 
-    use core::str::from_utf8;
     use super::*;
-
-    #[cfg(all(feature = "logs", feature = "std"))]
-    fn setup_logger() {
-        use std::sync::Once;
-        static INITIALIZE: Once = Once::new();
-        INITIALIZE.call_once(env_logger::init);
-    }
-
-    #[cfg(not(feature = "logs"))]
-    fn setup_logger() {}
+    use core::str::from_utf8;
 
     #[test]
     fn truncate() {
@@ -164,43 +179,44 @@ mod tests {
 
     #[test]
     fn shift_right() {
-        setup_logger();
-        let mut ls = CacheString::try_from_str("abcdefg").unwrap();
+        let _ = env_logger::try_init();
+        let mut ls = SmallString::try_from_str("abcdefg").unwrap();
         unsafe { shift_right_unchecked(&mut ls, 0, 4) };
-        ls.1 += 4;
+        ls.size += 4;
         assert_eq!(ls.as_str(), "abcdabcdefg");
     }
 
     #[test]
     fn shift_left() {
-        setup_logger();
-        let mut ls = CacheString::try_from_str("abcdefg").unwrap();
+        let _ = env_logger::try_init();
+        let mut ls = SmallString::try_from_str("abcdefg").unwrap();
         unsafe { shift_left_unchecked(&mut ls, 1, 0) };
-        ls.1 -= 1;
+        ls.size -= 1;
         assert_eq!(ls.as_str(), "bcdefg");
     }
 
     #[test]
     fn shift_nop() {
-        setup_logger();
-        let mut ls = CacheString::try_from_str("abcdefg").unwrap();
+        let _ = env_logger::try_init();
+        let mut ls = SmallString::try_from_str("abcdefg").unwrap();
         unsafe { shift_right_unchecked(&mut ls, 0, 0) };
         assert_eq!(ls.as_str(), "abcdefg");
         unsafe { shift_left_unchecked(&mut ls, 0, 0) };
         assert_eq!(ls.as_str(), "abcdefg");
     }
 
-
     #[test]
     fn encode_char_utf8() {
-        use array::Buffer;
-        setup_logger();
-        let mut string = CacheString::default();
+        let _ = env_logger::try_init();
+        let mut string = SmallString::default();
         unsafe { encode_char_utf8_unchecked(&mut string, 'a', 0) };
-        assert_eq!(from_utf8(unsafe { &string.buffer()[..1] }).unwrap(), "a");
+        assert_eq!(from_utf8(&string.array.as_mut_slice()[..1]).unwrap(), "a");
 
-        let mut string = CacheString::try_from_str("a").unwrap();
+        let mut string = SmallString::try_from_str("a").unwrap();
         unsafe { encode_char_utf8_unchecked(&mut string, '🤔', 1) };
-        assert_eq!(from_utf8(unsafe { &string.buffer()[..5] }).unwrap(), "a🤔");
+        assert_eq!(
+            from_utf8(&string.array.as_mut_slice()[..5]).unwrap(),
+            "a🤔"
+        );
     }
 }
