@@ -103,16 +103,6 @@ fn from_utf8() {
     );
 }
 
-#[test]
-#[ignore]
-/// TODO
-fn from_utf8_lossy() {
-    assert(
-        |s| String::from_utf8(s.as_bytes().to_vec()),
-        |s| MaxString::from_utf8_lossy(s.as_bytes()),
-    );
-}
-
 #[inline]
 fn invalidate_utf8(buf: &mut [u8]) -> &mut [u8] {
     if buf.len() >= 4 {
@@ -144,18 +134,6 @@ fn from_utf8_invalid() {
     }
 }
 
-#[test]
-#[ignore]
-/// TODO
-fn from_utf8_lossy_invalid() {
-    unsafe {
-        assert(
-            |s| String::from_utf8(invalidate_utf8(s.to_owned().as_bytes_mut()).to_vec()),
-            |s| MaxString::from_utf8_lossy(invalidate_utf8(s.to_owned().as_bytes_mut())),
-        );
-    }
-}
-
 #[test]
 fn try_from_utf16() {
     let utf16 = |s: &str| s.encode_utf16().collect::<Vec<_>>();
@@ -426,7 +404,7 @@ fn insert_unchecked() {
         },
         |s| {
             let mut ms = MaxString::try_from_str(s).unwrap();
-            is_inside_boundary(2, ms.len())
+            is_inside_boundary(2u8, ms.len())
                 .map_err(Error::from)
                 .and_then(|()| Ok(is_char_boundary(&ms, 2)?))
                 .map(|()| unsafe { ms.insert_unchecked(2, 'a') })
@@ -482,7 +460,7 @@ fn insert_str_unchecked() {
         },
         |s| {
             let mut ms = MaxString::try_from_str(s).unwrap();
-            is_inside_boundary(2, ms.len())
+            is_inside_boundary(2u8, ms.len())
                 .map_err(Error::from)
                 .and_then(|()| Ok(is_char_boundary(&ms, 2)?))
                 .map(|()| unsafe { ms.insert_str_unchecked(2, s) })
@@ -670,7 +648,7 @@ where
     F: Fn(&'static str) -> T + RefUnwindSafe,
     G: Fn(&'static str) -> U + RefUnwindSafe,
 {
-    setup_logger();
+    let _ = env_logger::try_init();
     for string in STRINGS.into_iter() {
         let f = f(string).normalize();
         let g = g(string).normalize();
